# -*- coding: utf-8 -*-
from pysummarization.tokenizable_doc import TokenizableDoc
import MeCab


class SimpleTokenizer(TokenizableDoc):
    '''
    Tokenize delimited sentence with a blank.
    '''
    
    def tokenize(self, sentence_str):
        '''
        Tokenize str.
        
        Args:
            sentence_str:   tokenized string.
        
        Returns:
            [token, token, token, ...]
        '''
        token_list = sentence_str.split(" ")
        return token_list



# -*- coding: utf-8 -*-
from pysummarization.similarity_filter import SimilarityFilter


class Jaccard(SimilarityFilter):
    '''
    Concrete class for filtering mutually similar sentences.
    '''

    def calculate(self, token_list_x, token_list_y):
        '''
        Calculate similarity with the Jaccard coefficient.
        
        Concrete method.
        
        Args:
            token_list_x:    [token, token, token, ...]
            token_list_y:    [token, token, token, ...]
        
        Returns:
            Similarity.
        '''

        x, y = self.unique(token_list_x, token_list_y)
        try:
            result = len(x & y) / len(x | y)
        except ZeroDivisionError:
            result = 0.0
        return result



# -*- coding: utf-8 -*-
import numpy as np
from pysummarization.abstractable_doc import AbstractableDoc


class StdAbstractor(AbstractableDoc):
    '''
    The filtering list of tokens with the difference between the standard deviation.
    '''

    def filter(self, scored_list):
        '''
        Filtering with std.

        Args:
            scored_list:    The list of scoring.

        Retruns:
            The list of filtered result.

        '''
        if len(scored_list) > 0:
            avg = np.mean([s[1] for s in scored_list])
            std = np.std([s[1] for s in scored_list])
        else:
            avg = 0
            std = 0
        limiter = avg + 0.5 * std
        mean_scored = [(sent_idx, score) for (sent_idx, score) in scored_list if score > limiter]
        return mean_scored



# -*- coding: utf-8 -*-
from abc import ABCMeta, abstractmethod
from pysummarization.tokenizable_doc import TokenizableDoc


class NlpBase(object):
    '''
    The base class for NLP.
    '''
    
    # object of tokenizer.
    __tokenizable_doc = None

    def get_tokenizable_doc(self):
        ''' getter '''
        if isinstance(self.__tokenizable_doc, TokenizableDoc):
            return self.__tokenizable_doc
        else:
            raise TypeError()
    
    def set_tokenizable_doc(self, value):
        ''' setter '''
        if isinstance(value, TokenizableDoc):
            self.__tokenizable_doc = value
        else:
            raise TypeError()

    tokenizable_doc = property(get_tokenizable_doc, set_tokenizable_doc)

    # Delimiter for self.listup_sentence.
    __delimiter_list=["。", "\n"]
    
    def get_delimiter_list(self):
        ''' getter '''
        return self.__delimiter_list

    def set_delimiter_list(self, value):
        ''' setter '''
        self.__delimiter_list = value

    delimiter_list = property(get_delimiter_list, set_delimiter_list)

    # List of tokens.
    __token = []

    def get_token(self):
        ''' getter '''
        return self.__token

    def set_token(self, value):
        ''' setter '''
        self.__token = value

    token = property(get_token, set_token)

    def tokenize(self, data):
        '''
        Tokenize sentence and set the list of tokens to self.token.

        Args:
            data:    string.

        '''
        self.token = self.tokenizable_doc.tokenize(data)

    def listup_sentence(self, data, counter=0):
        '''
        Divide string into sentence list.

        Args:
            data:               string.
            counter:            recursive counter.

        Returns:
            List of sentences.

        '''
        delimiter = self.delimiter_list[counter]
        sentence_list = []
        [sentence_list.append(sentence + delimiter) for sentence in data.split(delimiter) if sentence != ""]
        if counter + 1 < len(self.delimiter_list):
            sentence_list_r = []
            [sentence_list_r.extend(self.listup_sentence(sentence, counter+1)) for sentence in sentence_list]
            sentence_list = sentence_list_r

        return sentence_list



# -*- coding: utf-8 -*-
from abc import ABCMeta, abstractmethod


class AbstractableDoc(metaclass=ABCMeta):
    '''
    Automatic abstraction and summarization.
    This is the filtering approach.

    Reference.
        - Luhn, Hans Peter. "The automatic creation of literature abstracts."  IBM Journal of research and development 2.2 (1958): 159-165.
        - http://www.oreilly.co.jp/books/9784873116792/

    This `interface` is designed the `Strategy Pattern`.
    '''

    @abstractmethod
    def filter(self, scored_list):
        '''
        Execute filtering sentences.

        Args:
            scored_list:    The list of statistical information derived from word frequency and distribution.

        Retruns:
            the list of filtered sentence.

        '''
        raise NotImplementedError("This method must be implemented.")



# -*- coding: utf-8 -*-
import numpy as np
from pysummarization.similarity_filter import SimilarityFilter
from pysummarization.vectorlizabletoken.tfidf_vectorizer import TfidfVectorizer


class TfIdfCosine(SimilarityFilter):
    '''
    Concrete class for filtering mutually similar sentences.
    '''

    def calculate(self, token_list_x, token_list_y):
        '''
        Calculate similarity with the so-called Cosine similarity of Tf-Idf vectors.
        
        Concrete method.
        
        Args:
            token_list_x:    [token, token, token, ...]
            token_list_y:    [token, token, token, ...]
        
        Returns:
            Similarity.
        '''
        if len(token_list_x) == 0 or len(token_list_y) == 0:
            return 0.0

        document_list = token_list_x.copy()
        [document_list.append(v) for v in token_list_y]
        document_list = list(set(document_list))

        tfidf_vectorizer = TfidfVectorizer(document_list)

        vector_list_x = tfidf_vectorizer.vectorlize(token_list_x)
        vector_list_y = tfidf_vectorizer.vectorlize(token_list_y)
        
        if len(vector_list_x) > len(vector_list_y):
            [vector_list_y.append(0.0) for _ in range(len(vector_list_x) - len(vector_list_y))]
        elif len(vector_list_y) > len(vector_list_x):
            [vector_list_x.append(0.0) for _ in range(len(vector_list_y) - len(vector_list_x))]

        dot_prod = np.dot(vector_list_x, vector_list_y)
        norm_x = np.linalg.norm(vector_list_x)
        norm_y = np.linalg.norm(vector_list_y)
        try:
            result = dot_prod / (norm_x * norm_y)
            if np.isnan(result) is True:
                return 0.0
            else:
                return result
        except ZeroDivisionError:
            return 0.0



# -*- coding: utf-8 -*-
import nltk
from pysummarization.vectorlizable_token import VectorlizableToken


class TfidfVectorizer(VectorlizableToken):
    '''
    Vectorlize token.
    '''
    # Document
    __collection = []
    
    def __init__(self, token_list_list):
        '''
        Initialize.
        
        Args:
            token_list_list:    The list of list of tokens.
        '''
        self.__collection = nltk.TextCollection(token_list_list)

    def vectorlize(self, token_list):
        '''
        Tokenize token list.
        
        Args:
            token_list:   The list of tokens..
        
        Returns:
            [vector of token, vector of token, vector of token, ...]
        '''
        vector_list = [self.__collection.tf_idf(token, self.__collection) for token in list(set(token_list))]
        return vector_list



# -*- coding: utf-8 -*-
from pysummarization.nlpbase.auto_abstractor import AutoAbstractor
from pysummarization.n_gram import Ngram


class NgramAutoAbstractor(AutoAbstractor):
    '''
    The object for automatic summarization.
    The minimum unit of token is N-gram.
    '''
    
    # The object of N-gram.
    __n_gram = None
    
    def get_n_gram(self):
        ''' gettter '''
        if isinstance(self.__n_gram, Ngram):
            return self.__n_gram
        else:
            raise TypeError("The type of n_gram must be Ngram.")

    def set_n_gram(self, value):
        ''' setter '''
        if isinstance(value, Ngram):
            self.__n_gram = value
        else:
            raise TypeError("The type of n_gram must be Ngram.")

    n_gram = property(get_n_gram, set_n_gram)

    # N of N-gram.
    __n = 2
    
    def get_n(self):
        ''' getter '''
        if isinstance(self.__n, int):
            return self.__n
        else:
            raise TypeError("The type of n must be int.")

    def set_n(self, value):
        ''' setter '''
        if isinstance(value, int):
            self.__n = value
        else:
            raise TypeError("The type of n must be int.")

    n = property(get_n, set_n)

    def tokenize(self, data):
        '''
        Tokenize sentence.

        Args:
            [n-gram, n-gram, n-gram, ...]

        '''
        super().tokenize(data)
        token_tuple_zip = self.n_gram.generate_tuple_zip(self.token, self.n)
        token_list = []
        self.token = ["".join(list(token_tuple)) for token_tuple in token_tuple_zip]



import nltk
from pysummarization.nlp_base import NlpBase
from pysummarization.abstractable_doc import AbstractableDoc
from pysummarization.similarity_filter import SimilarityFilter

class AutoAbstractor(NlpBase):
    '''
    The object for automatic summarization.
    '''

    # Only top-n scored tokens must be set to target. 
    __target_n = 100

    def get_target_n(self):
        ''' getter '''
        if isinstance(self.__target_n, int) is False:
            raise TypeError("The type of __target_n must be int.")
        return self.__target_n

    def set_target_n(self, value):
        ''' setter '''
        if isinstance(value, int) is False:
            raise TypeError("The type of __target_n must be int.")
        self.__target_n = value

    target_n = property(get_target_n, set_target_n)

    # Adjacent distance.
    __cluster_threshold = 5

    def get_cluster_threshold(self):
        ''' getter '''
        if isinstance(self.__cluster_threshold, int) is False:
            raise TypeError("The type of __cluster_threshold must be int.")
        return self.__cluster_threshold

    def set_cluster_threshold(self, value):
        ''' setter '''
        if isinstance(value, int) is False:
            raise TypeError("The type of __cluster_threshold must be int.")
        self.__cluster_threshold = value

    cluster_threshold = property(get_cluster_threshold, set_cluster_threshold)

    # The number of returned sentences.
    __top_sentences = 5

    def get_top_sentences(self):
        ''' getter '''
        if isinstance(self.__top_sentences, int) is False:
            raise TypeError("The type of __top_sentences must be int.")
        return self.__top_sentences

    def set_top_sentences(self, value):
        ''' setter '''
        if isinstance(value, int) is False:
            raise TypeError("The type of __top_sentences must be int.")
        self.__top_sentences = value

    top_sentences = property(get_top_sentences, set_top_sentences)

    def summarize(self, document, Abstractor, similarity_filter=None):
        '''
        Execute summarization.

        Args:
            document:           The target document.
            Abstractor:         The object of AbstractableDoc.
            similarity_filter   The object of SimilarityFilter.

        Returns:
            dict{
                "summarize_result": The list of summarized sentences., 
                "scoring_data":     The list of scores.
            }
        '''
        if isinstance(document, str) is False:
            raise TypeError("The type of document must be str.")

        if isinstance(Abstractor, AbstractableDoc) is False:
            raise TypeError("The type of Abstractor must be AbstractableDoc.")

        if isinstance(similarity_filter, SimilarityFilter) is False and similarity_filter is not None:
            raise TypeError("The type of similarity_filter must be SimilarityFilter.")

        normalized_sentences = self.listup_sentence(document)

        # for filtering similar sentences.
        if similarity_filter is not None:
            normalized_sentences = similarity_filter.similar_filter_r(normalized_sentences)

        self.tokenize(document)
        words = self.token

        fdist = nltk.FreqDist(words)
        top_n_words = [w[0] for w in fdist.items()][:self.target_n]
        scored_list = self.__closely_associated_score(normalized_sentences, top_n_words)
        filtered_list = Abstractor.filter(scored_list)
        result_list = [normalized_sentences[idx] for (idx, score) in filtered_list]
        result_dict = {
            "summarize_result": result_list,
            "scoring_data": filtered_list
        }
        return result_dict

    def __closely_associated_score(self, normalized_sentences, top_n_words):
        '''
        Scoring the sentence with closely associations.

        Args:
            normalized_sentences:   The list of sentences.
            top_n_words:            Important sentences.

        Returns:
            The list of scores.
        '''
        scores_list = []
        sentence_idx = -1

        for sentence in normalized_sentences:
            self.tokenize(sentence)
            sentence = self.token

            sentence_idx += 1
            word_idx = []

            for w in top_n_words:
                try:
                    word_idx.append(sentence.index(w))
                except ValueError:
                    pass

            word_idx.sort()

            if len(word_idx) == 0:
                continue

            clusters = []
            cluster = [word_idx[0]]
            i = 1
            while i < len(word_idx):
                if word_idx[i] - word_idx[i - 1] < self.cluster_threshold:
                    cluster.append(word_idx[i])
                else:
                    clusters.append(cluster[:])
                    cluster = [word_idx[i]]
                i += 1
            clusters.append(cluster)

            max_cluster_score = 0
            for c in clusters:
                significant_words_in_cluster = len(c)
                total_words_in_cluster = c[-1] - c[0] + 1
                score = 1.0 * significant_words_in_cluster \
                    * significant_words_in_cluster / total_words_in_cluster

                if score > max_cluster_score:
                    max_cluster_score = score

            scores_list.append((sentence_idx, score))

        return scores_list



# -*- coding: utf-8 -*-
from abc import ABCMeta, abstractmethod


class VectorlizableToken(metaclass=ABCMeta):
    '''
    Vectorlize token.
    '''
    
    @abstractmethod
    def vectorlize(self, token_list):
        '''
        Tokenize token list.
        
        Args:
            token_list:   The list of tokens.
        
        Returns:
            [vector of token, vector of token, vector of token, ...]
        '''
        raise NotImplementedError("This method must be implemented.")



# -*- coding: utf-8 -*-
from abc import ABCMeta, abstractmethod


class TokenizableDoc(metaclass=ABCMeta):
    '''
    Tokenize string.
    
    '''
    
    @abstractmethod
    def tokenize(self, sentence_str):
        '''
        Tokenize str.
        
        Args:
            sentence_str:   tokenized string.
        
        Returns:
            [token, token, token, ...]
        '''
        raise NotImplementedError("This method must be implemented.")



# -*- coding: utf-8 -*-
from pysummarization.similarity_filter import SimilarityFilter


class Tanimoto(SimilarityFilter):
    '''
    Concrete class for filtering mutually similar sentences.
    '''

    def calculate(self, token_list_x, token_list_y):
        '''
        Calculate similarity with the Tanimoto coefficient.
        
        Concrete method.
        
        Args:
            token_list_x:    [token, token, token, ...]
            token_list_y:    [token, token, token, ...]
        
        Returns:
            Similarity.
        '''
        match_list = [tanimoto_value for tanimoto_value in token_list_x if tanimoto_value in token_list_y]
        return float(len(match_list) / (len(token_list_x) + len(token_list_y) - len(match_list)))



# -*- coding: utf-8 -*-
from abc import ABCMeta, abstractmethod
from pysummarization.nlp_base import NlpBase


class SimilarityFilter(metaclass=ABCMeta):
    '''
    Abstract class for filtering mutually similar sentences.
    '''

    # NlpBase
    __nlp_base = None

    def get_nlp_base(self):
        ''' getter '''
        if isinstance(self.__nlp_base, NlpBase) is False:
            raise TypeError("The type of self.__nlp_base must be NlpBase.")

        return self.__nlp_base

    def set_nlp_base(self, value):
        ''' setter '''
        if isinstance(value, NlpBase) is False:
            raise TypeError("The type of value must be NlpBase.")
        
        self.__nlp_base = value

    nlp_base = property(get_nlp_base, set_nlp_base)

    # Cut off threshold.
    __similarity_limit = 0.8

    def get_similarity_limit(self):
        ''' getter '''
        if isinstance(self.__similarity_limit, float) is False:
            raise TypeError("__similarity_limit must be float.")
        return self.__similarity_limit

    def set_similarity_limit(self, value):
        ''' setter '''
        if isinstance(value, float) is False:
            raise TypeError("__similarity_limit must be float.")
        self.__similarity_limit = value

    similarity_limit = property(get_similarity_limit, set_similarity_limit)

    @abstractmethod
    def calculate(self, token_list_x, token_list_y):
        '''
        Calculate similarity.
        
        Abstract method.
        
        Args:
            token_list_x:    [token, token, token, ...]
            token_list_y:    [token, token, token, ...]
        
        Returns:
            Similarity.
        '''
        raise NotImplementedError("This method must be implemented.")

    def unique(self, token_list_x, token_list_y):
        '''
        Remove duplicated elements.
        
        Args:
            token_list_x:    [token, token, token, ...]
            token_list_y:    [token, token, token, ...]

        Returns:
            Tuple(token_list_x, token_list_y)
        '''
        x = set(list(token_list_x))
        y = set(list(token_list_y))
        return (x, y)

    def count(self, token_list):
        '''
        Count the number of tokens in `token_list`.
        
        Args:
            token_list:    The list of tokens.

        Returns:
            {token: the numbers}
        '''
        token_dict = {}
        for token in token_list:
            if token in token_dict:
                token_dict[token] += 1
            else:
                token_dict[token] = 1
        return token_dict

    def similar_filter_r(self, sentence_list):
        '''
        Filter mutually similar sentences.
        
        Args:
            sentence_list:    The list of sentences.

        Returns:
            The list of filtered sentences.
        '''
        result_list = []
        recursive_list = []

        try:
            self.nlp_base.tokenize(sentence_list[0])
            subject_token = self.nlp_base.token
            result_list.append(sentence_list[0])
            if len(sentence_list) > 1:
                for i in range(len(sentence_list)):
                    if i > 0:
                        self.nlp_base.tokenize(sentence_list[i])
                        object_token = self.nlp_base.token
                        similarity = self.calculate(subject_token, object_token)
                        if similarity <= self.similarity_limit:
                            recursive_list.append(sentence_list[i])

            if len(recursive_list) > 0:
                result_list.extend(self.similar_filter_r(recursive_list))
        except IndexError:
            result_list = sentence_list

        return result_list



#!/user/bin/env python
# -*- coding: utf-8 -*-
from abc import ABCMeta, abstractmethod


class ReadableWebPDF(metaclass=ABCMeta):
    '''
    Read strings in PDF documents
    '''

    @abstractmethod
    def url_to_text(self, url):
        '''
        Transform PDF documents to strings.

        Args:
            url:   URL

        Returns:
            string.

        '''
        raise NotImplementedError("This method must be implemented.")

    @abstractmethod
    def is_pdf_url(self, url):
        '''
        Check PDF format.

        Args:
            url:    URL

        Returns:
            True: PDF, False: not PDF
        '''
        raise NotImplementedError("This method must be implemented.")



from io import StringIO
from pdfminer.pdfpage import PDFPage
from pdfminer.pdfinterp import PDFResourceManager
from pdfminer.pdfinterp import PDFPageInterpreter
from pdfminer.converter import TextConverter
from pdfminer.layout import LAParams
import urllib.request
from pysummarization.readable_web_pdf import ReadableWebPDF


class WebPDFReading(ReadableWebPDF):
    '''
    Read the PDF.
    '''

    def url_to_text(self, url):
        '''
        Download PDF file and transform its document to string.

        Args:
            url:   PDF url.

        Returns:
            string.

        '''
        path, headers = urllib.request.urlretrieve(url)
        return self.path_to_text(path)

    def path_to_text(self, path):
        '''
        Transform local PDF file to string.

        Args:
            path:   path to PDF file.

        Returns:
            string.

        '''
        rsrcmgr = PDFResourceManager()
        retstr = StringIO()
        codec = 'utf-8'
        laparams = LAParams()
        device = TextConverter(rsrcmgr, retstr, codec=codec, laparams=laparams)
        fp = open(path, 'rb')
        interpreter = PDFPageInterpreter(rsrcmgr, device)
        password = ""
        maxpages = 0
        caching = True
        pagenos = set()

        pages_data = PDFPage.get_pages(
            fp,
            pagenos,
            maxpages=maxpages,
            password=password,
            caching=caching,
            check_extractable=True
        )

        for page in pages_data:
            interpreter.process_page(page)

        text = retstr.getvalue()
        text = text.replace("\n", "")

        fp.close()
        device.close()
        retstr.close()
        return text

    def is_pdf_url(self, url):
        '''
        Check PDF file format.
        
        @TODO(chimera0): validation.

        Args:
            url:    URL

        Returns:
            True: PDF, False: not PDF
        '''
        if url[-4:] == ".pdf":
            return True
        else:
            return False

if __name__ == "__main__":
    import sys
    url = sys.argv[1]
    text = WebPDFReading().url_to_text(url)
    print(text[:300])



#!/user/bin/env python
# -*- coding: utf-8 -*-


class Ngram(object):
    '''
    N-gram
    '''

    def generate_ngram_data_set(self, token_list, n=2):
        '''
        Generate the N-gram's pair.

        Args:
            token_list:     The list of tokens.
            n               N

        Returns:
            zip of Tuple(Training N-gram data, Target N-gram data)
        '''
        n_gram_tuple_zip = self.generate_tuple_zip(token_list, n)
        n_gram_tuple_list = [n_gram_tuple for n_gram_tuple in n_gram_tuple_zip]
        n_gram_data_set = self.generate_tuple_zip(n_gram_tuple_list, 2)
        return n_gram_data_set

    def generate_skip_gram_data_set(self, token_list):
        '''
        Generate the Skip-gram's pair.

        Args:
            token_list:     The list of tokens.

        Returns:
            zip of Tuple(Training N-gram data, Target N-gram data)
        '''
        n_gram_tuple_zip = self.generate_tuple_zip(token_list, 3)
        skip_gram_list = []
        for pre, point, post in n_gram_tuple_zip:
            skip_gram_list.append((point, pre))
            skip_gram_list.append((point, post))
        return zip(skip_gram_list)

    def generate_tuple_zip(self, token_list, n=2):
        '''
        Generate the N-gram.

        Args:
            token_list:     The list of tokens.
            n               N

        Returns:
            zip of Tuple(N-gram)
        '''
        return zip(*[token_list[i:] for i in range(n)])



# -*- coding: utf-8 -*-
from pysummarization.readable_web_pdf import ReadableWebPDF
from time import sleep
import urllib
from pyquery import PyQuery as pq


class WebScraping(object):
    '''
    Object of Web-scraping.

    This is only a demo.
    '''

    # List of scraped dom objects.
    __dom_object_list = ["body"]
    # List of not scraped dom objects.
    __remove_object_list = ["script", "style"]

    # Object of ReadableWebPdf.
    __readable_web_pdf = None

    def get_readable_web_pdf(self):
        ''' getter '''
        if isinstance(self.__readable_web_pdf, ReadableWebPDF) is False and self.__readable_web_pdf is not None:
            raise TypeError("The type of __readable_web_pdf must be ReadableWebPDF.")
        return self.__readable_web_pdf

    def set_readable_web_pdf(self, value):
        ''' setter '''
        if isinstance(value, ReadableWebPDF) is False and value is not None:
            raise TypeError("The type of __readable_web_pdf must be ReadableWebPDF.")
        self.__readable_web_pdf = value

    readable_web_pdf = property(get_readable_web_pdf, set_readable_web_pdf)

    def scrape(self, url):
        '''
        Execute Web-Scraping.
        The target dom objects are in self.__dom_object_list.

        Args:
            url:    Web site url.

        Returns:
            The result. this is a string.

        @TODO(chimera0): check URLs format.
        '''
        if isinstance(url, str) is False:
            raise TypeError("The type of url must be str.")

        if self.readable_web_pdf is not None and self.readable_web_pdf.is_pdf_url(url) is True:
            web_data = self.readable_web_pdf.url_to_text(url)
        else:
            web_data = ""
            req = urllib.request.Request(url=url)
            with urllib.request.urlopen(req) as f:
                web = f.read().decode('utf-8')
                dom = pq(web)
                [dom(remove_object).remove() for remove_object in self.__remove_object_list]

                for dom_object in self.__dom_object_list:
                    web_data += dom(dom_object).text()

        sleep(1)
        return web_data



# -*- coding: utf-8 -*-
from pysummarization.abstractable_doc import AbstractableDoc


class TopNRankAbstractor(AbstractableDoc):
    '''
    Ranking the list of tokens.
    '''

    # N of top-n.
    __top_n = 10

    def get_top_n(self):
        ''' getter '''
        if isinstance(self.__top_n, int) is False:
            raise TypeError("The type of __top_n must be int.")
        return self.__top_n

    def set_top_n(self, value):
        ''' setter '''
        if isinstance(value, int) is False:
            raise TypeError("The type of __top_n must be int.")
        self.__top_n = value

    top_n = property(get_top_n, set_top_n)

    def filter(self, scored_list):
        '''
        Filtering with top-n ranking.

        Args:
            scored_list:    The list of scoring.

        Retruns:
            The list of filtered result.

        '''
        top_n_key = -1 * self.top_n
        top_n_list = sorted(scored_list, key=lambda x: x[1])[top_n_key:]
        result_list = sorted(top_n_list, key=lambda x: x[0])
        return result_list
